package testObjectDb;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerEditor;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationEvent;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationStrategy;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TableViewerEditor;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.part.ViewPart;

public class View extends ViewPart {

	private TableViewer viewer;
	private CellEditor editor;
	private ArrayList<Pilot>pilots;
	private EntityManagerFactory emf;
	
	/**
	 * This is a callback that will allow us to create the viewer and initialise
	 * it.
	 */
	public void createPartControl(Composite parent) {
		emf = Persistence.createEntityManagerFactory("$objectdb/db/pilots.odb");
		EntityManager em = emf.createEntityManager();
        em.getTransaction().begin();

			Pilot pilot = new Pilot("Rubens Barrichello", 99);
			em.persist(pilot);
			Pilot pilot2 = new Pilot("Rubens Barrichello", 99);
			em.persist(pilot2);

		em.getTransaction().commit();
		TypedQuery<Pilot> query =
            em.createQuery("SELECT p FROM Point p", Pilot.class);
        List<Pilot> results = query.getResultList();
	        pilots = new ArrayList<Pilot>();
	        pilots.addAll(results);
		em.clear();
		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		final ColumnViewerEditorActivationStrategy actSupport = new ColumnViewerEditorActivationStrategy(viewer) {
			protected boolean isEditorActivationEvent( ColumnViewerEditorActivationEvent event) {
				return (event.eventType == ColumnViewerEditorActivationEvent.TRAVERSAL)
				|| (event.eventType == ColumnViewerEditorActivationEvent.MOUSE_CLICK_SELECTION)
				|| ((event.eventType == ColumnViewerEditorActivationEvent.KEY_PRESSED) && (event.keyCode == SWT.CR))
				|| (event.eventType == ColumnViewerEditorActivationEvent.PROGRAMMATIC);
			}
		};

		TableViewerEditor.create(viewer, actSupport,
				ColumnViewerEditor.TABBING_HORIZONTAL
				| ColumnViewerEditor.TABBING_MOVE_TO_ROW_NEIGHBOR
				| ColumnViewerEditor.TABBING_VERTICAL
				| ColumnViewerEditor.KEYBOARD_ACTIVATION);
		
		TableViewerColumn nameColumn = new TableViewerColumn(viewer, SWT.NONE);
		nameColumn.getColumn().setWidth(300);
		nameColumn.getColumn().setText("Name");
		nameColumn.setLabelProvider(new ColumnLabelProvider() {
			public String getText(Object element) {
				if(((Pilot) element).getName()==null) {
					return "";
				}else {
					return ((Pilot)element).getName();}
			}
		});
		
		nameColumn.setEditingSupport(new EditingSupport(viewer) {
			protected boolean canEdit(Object arg0) {
				return true;
			}

			protected CellEditor getCellEditor(Object arg0) {
				editor = new TextCellEditor(((TableViewer) getViewer()).getTable());
				editor.getControl().addListener(SWT.Traverse, new Listener(){
					public void handleEvent( Event event ) {
						if(event.detail == SWT.TRAVERSE_RETURN){
							event.detail = SWT.TRAVERSE_TAB_NEXT;
						}
					}
				});
				editor.getControl().addKeyListener(new KeyAdapter() {
					public void keyPressed(KeyEvent arg0) {
						if(arg0.keyCode == SWT.DEL) {
							int intRows[] = viewer.getTable().getSelectionIndices();
							int rowCount = intRows.length;
							if(rowCount > 1) {
								while (rowCount - 1 >= 0 ) {
									Object ob = viewer.getElementAt(intRows[rowCount - 1]);
									Pilot p = (Pilot) ob;
									viewer.remove(p);
									rowCount--;
								}
								return;
							}
						}
					}
				});
				return editor;
			}

			protected Object getValue(Object arg0) {
				Pilot p = (Pilot) arg0;
				if (p.getName() != null) { return p.getName(); }
				return "";
			}

			protected void setValue(Object element, Object value) {
				if(value instanceof String && ((String)value).trim().equals("")){return;}
				Pilot p = (Pilot) element;

				if(!pilots.contains(p)) {
					pilots.add(p);
				}
				if (p.getName()==null) {
					p.setName((String) value);
				}else if (p.getName().equals((value))) {
					return; // if content stays the same do nothing
				} else {
					p.setName((String) value);
				}
				viewer.update(element, null);				
			}   		
		});
		
		TableViewerColumn pointsColumn = new TableViewerColumn(viewer, SWT.NONE);
		pointsColumn.getColumn().setWidth(100);
		pointsColumn.getColumn().setText("Points");
		pointsColumn.setLabelProvider(new ColumnLabelProvider() {
			public String getText(Object element) {
				if(((Pilot) element).getPoints() == 0) {return "";
				}else {return Integer.toString(((Pilot)element).getPoints());}
			}
		});
		
		pointsColumn.setEditingSupport(new EditingSupport(viewer) {
			protected boolean canEdit(Object arg0) {
				return true;
			}

			protected CellEditor getCellEditor(Object arg0) {
				editor = new TextCellEditor(((TableViewer) getViewer()).getTable());
				editor.getControl().addListener(SWT.Traverse, new Listener(){
					public void handleEvent( Event event ) {
						if(event.detail == SWT.TRAVERSE_RETURN){
							event.detail = SWT.TRAVERSE_TAB_NEXT;
						}
					}
				});
				editor.getControl().addKeyListener(new KeyAdapter() {
					public void keyPressed(KeyEvent arg0) {
						if(arg0.keyCode == SWT.DEL) {
							int intRows[] = viewer.getTable().getSelectionIndices();
							int rowCount = intRows.length;
							if(rowCount > 1) {
								while (rowCount - 1 >= 0 ) {
									Object ob = viewer.getElementAt(intRows[rowCount - 1]);
									Pilot p = (Pilot) ob;
									viewer.remove(p);
									rowCount--;
								}
								return;
							}
						}
					}
				});
				return editor;
			}

			protected Object getValue(Object arg0) {
				Pilot p = (Pilot) arg0;
				if (p.getPoints() >= 0) { return p.getPoints(); }
				return "";
			}

			protected void setValue(Object element, Object value) {
				if(value instanceof String && ((String)value).trim().equals("")){return;}
				Pilot p = (Pilot) element;

				if(!pilots.contains(p)) {
					pilots.add(p);
				}
				if (p.getPoints() >= 0) {
					p.setPoints(Integer.parseInt(String.valueOf(value)));
				}else if (p.getPoints() == Integer.parseInt(String.valueOf(value))) {return; // if content stays the same do nothing
				} else {p.setPoints(Integer.parseInt(String.valueOf(value)));}
				viewer.update(element, null);				
			}   		
		});
		
		viewer.setContentProvider(new TableContentProvider());
		// Provide the input to the ContentProvider
		viewer.setInput(pilots);
		viewer.getTable().setLinesVisible(true);
		viewer.getTable().setHeaderVisible(true);
		viewer.getTable().addDisposeListener(new DisposeListener(){
			public void widgetDisposed(DisposeEvent e) {
				emf.close();
			}});	
	}
	
	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	}

	